@@ -1,20 +1,10 @@
+// SPDX-License-Identifier: LGPL-2.1-or-later
 /*
  * libiio - Library for interfacing industrial I/O (IIO) devices
  *
  * Copyright (C) 2014 Analog Devices, Inc.
  * Author: Paul Cercueil <paul.cercueil@analog.com>
- *
- * This library is free software; you can redistribute it and/or
- * modify it under the terms of the GNU Lesser General Public
- * License as published by the Free Software Foundation; either
- * version 2.1 of the License, or (at your option) any later version.
- *
- * This library is distributed in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
- * Lesser General Public License for more details.
- *
- * */
+ */
 
 #include "debug.h"
 #include "iio-private.h"
@@ -100,6 +90,7 @@ static const char * const modifier_names[] = {
 	[IIO_MOD_CO2] = "co2",
 	[IIO_MOD_ETHANOL] = "ethanol",
 	[IIO_MOD_H2] = "h2",
+	[IIO_MOD_O2] = "o2",
 	[IIO_MOD_VOC] = "voc",
 	[IIO_MOD_PM1] = "pm1",
 	[IIO_MOD_PM2P5] = "pm2p5",
@@ -176,168 +167,92 @@ void iio_channel_init_finalize(struct iio_channel *chn)
 	}
 }
 
-static char *get_attr_xml(struct iio_channel_attr *attr, size_t *length)
+static ssize_t iio_snprintf_chan_attr_xml(char *str, ssize_t len,
+					  struct iio_channel_attr *attr)
 {
-	char *str;
-	size_t len;
+	ssize_t ret, alen = 0;
 
-	len = strnlen(attr->name, MAX_ATTR_NAME);
-	len += sizeof("<attribute name=\"\" />") - 1;
+	if (!attr->filename)
+		return iio_snprintf(str, len, "<attribute name=\"%s\" />", attr->name);
 
-	if (attr->filename) {
-		len += strnlen(attr->filename, NAME_MAX);
-		len += sizeof(" filename=\"\"") - 1;
-	}
+	ret = iio_snprintf(str, len, "<attribute name=\"%s\" ", attr->name);
+	if (ret < 0)
+		return ret;
 
-	*length = len; /* just the chars */
-	len++;         /* room for terminating NULL */
-	str = malloc(len);
-	if (!str)
-		return NULL;
+	iio_update_xml_indexes(ret, &str, &len, &alen);
 
-	if (attr->filename)
-		iio_snprintf(str, len, "<attribute name=\"%s\" filename=\"%s\" />",
-				attr->name, attr->filename);
-	else
-		iio_snprintf(str, len, "<attribute name=\"%s\" />", attr->name);
+	ret = iio_xml_print_and_sanitized_param(str, len, "filename=\"",
+						attr->filename, "\" />");
+	if (ret < 0)
+		return ret;
 
-	return str;
+	return alen + ret;
 }
 
-static char * get_scan_element(const struct iio_channel *chn, size_t *length)
+static ssize_t iio_snprintf_scan_element_xml(char *str, ssize_t len,
+					     const struct iio_channel *chn)
 {
-	char buf[1024], repeat[12] = "", *str;
 	char processed = (chn->format.is_fully_defined ? 'A' - 'a' : 0);
+	char repeat[12] = "", scale[48] = "";
 
 	if (chn->format.repeat > 1)
 		iio_snprintf(repeat, sizeof(repeat), "X%u", chn->format.repeat);
 
-	iio_snprintf(buf, sizeof(buf), "<scan-element index=\"%li\" "
-			"format=\"%ce:%c%u/%u%s&gt;&gt;%u\" />",
+	if (chn->format.with_scale)
+		iio_snprintf(scale, sizeof(scale), "scale=\"%f\" ", chn->format.scale);
+
+	return iio_snprintf(str, len,
+			"<scan-element index=\"%li\" format=\"%ce:%c%u/%u%s&gt;&gt;%u\" %s/>",
 			chn->index, chn->format.is_be ? 'b' : 'l',
 			chn->format.is_signed ? 's' + processed : 'u' + processed,
 			chn->format.bits, chn->format.length, repeat,
-			chn->format.shift);
-
-	if (chn->format.with_scale) {
-		char *ptr = strrchr(buf, '\0');
-		iio_snprintf(ptr - 2, buf + sizeof(buf) - ptr + 2,
-				"scale=\"%f\" />", chn->format.scale);
-	}
-
-	str = iio_strdup(buf);
-	if (str)
-		*length = strlen(str);
-	return str;
+			chn->format.shift, scale);
 }
 
-/* Returns a string containing the XML representation of this channel */
-char * iio_channel_get_xml(const struct iio_channel *chn, size_t *length)
+ssize_t iio_snprintf_channel_xml(char *ptr, ssize_t len,
+				 const struct iio_channel *chn)
 {
-	ssize_t len;
-	char *ptr, *eptr, *str, **attrs, *scan_element = NULL;
-	size_t *attrs_len, scan_element_len = 0;
+	ssize_t ret, alen = 0;
 	unsigned int i;
 
-	len = sizeof("<channel id=\"\" type=\"\" ></channel>") - 1;
-	len += strnlen(chn->id, MAX_CHN_ID);
-	len += (chn->is_output ? sizeof("output") : sizeof("input")) - 1;
+
+	ret = iio_xml_print_and_sanitized_param(ptr, len, "<channel id=\"",
+						chn->id, "\"");
+	if (ret < 0)
+		return ret;
+	iio_update_xml_indexes(ret, &ptr, &len, &alen);
+
 	if (chn->name) {
-		len += sizeof(" name=\"\"") - 1;
-		len += strnlen(chn->name, MAX_CHN_NAME);
+		ret = iio_snprintf(ptr, len, " name=\"%s\"", chn->name);
+		if (ret < 0)
+			return ret;
+		iio_update_xml_indexes(ret, &ptr, &len, &alen);
 	}
 
+	ret = iio_snprintf(ptr, len, " type=\"%s\" >", chn->is_output ? "output" : "input");
+	if (ret < 0)
+		return ret;
+	iio_update_xml_indexes(ret, &ptr, &len, &alen);
+
 	if (chn->is_scan_element) {
-		scan_element = get_scan_element(chn, &scan_element_len);
-		if (!scan_element)
-			return NULL;
-		else
-			len += scan_element_len;
-	}
-
-	attrs_len = malloc(chn->nb_attrs * sizeof(*attrs_len));
-	if (!attrs_len)
-		goto err_free_scan_element;
-
-	attrs = malloc(chn->nb_attrs * sizeof(*attrs));
-	if (!attrs)
-		goto err_free_attrs_len;
-
-	for (i = 0; i < chn->nb_attrs; i++) {
-		char *xml = get_attr_xml(&chn->attrs[i], &attrs_len[i]);
-		if (!xml)
-			goto err_free_attrs;
-		attrs[i] = xml;
-		len += attrs_len[i];
-	}
-
-	len++;  /* room for terminating NULL */
-	str = malloc(len);
-	if (!str)
-		goto err_free_attrs;
-	ptr = str;
-	eptr = str + len;
-
-	if (len > 0) {
-		ptr += iio_snprintf(str, len, "<channel id=\"%s\"", chn->id);
-		len = eptr - ptr;
-	}
-
-	if (chn->name && len > 0) {
-		ptr += iio_snprintf(ptr, len, " name=\"%s\"", chn->name);
-		len = eptr - ptr;
-	}
-
-	if (len > 0) {
-		ptr += iio_snprintf(ptr, len, " type=\"%s\" >", chn->is_output ? "output" : "input");
-		len = eptr - ptr;
-	}
-
-	if (chn->is_scan_element && len > (ssize_t) scan_element_len) {
-		memcpy(ptr, scan_element, scan_element_len); /* Flawfinder: ignore */
-		ptr += scan_element_len;
-		len -= scan_element_len;
+		ret = iio_snprintf_scan_element_xml(ptr, len, chn);
+		if (ret < 0)
+			return ret;
+		iio_update_xml_indexes(ret, &ptr, &len, &alen);
 	}
 
 	for (i = 0; i < chn->nb_attrs; i++) {
-		if (len > (ssize_t) attrs_len[i]) {
-			memcpy(ptr, attrs[i], attrs_len[i]); /* Flawfinder: ignore */
-			ptr += attrs_len[i];
-			len -= attrs_len[i];
-		}
-		free(attrs[i]);
+		ret = iio_snprintf_chan_attr_xml(ptr, len, &chn->attrs[i]);
+		if (ret < 0)
+			return ret;
+		iio_update_xml_indexes(ret, &ptr, &len, &alen);
 	}
 
-	free(scan_element);
-	free(attrs);
-	free(attrs_len);
+	ret = iio_snprintf(ptr, len, "</channel>");
+	if (ret < 0)
+		return ret;
 
-	if (len > 0) {
-		ptr += iio_strlcpy(ptr, "</channel>", len);
-		len -= sizeof("</channel>") -1;
-	}
-
-	*length = ptr - str;
-
-	/* NULL char should be left, and that is it */
-	if (len != 1) {
-		IIO_ERROR("Internal libIIO error: iio_channel_get_xml str length issue\n");
-		free(str);
-		return NULL;
-	}
-
-	return str;
-
-err_free_attrs:
-	while (i--)
-		free(attrs[i]);
-	free(attrs);
-err_free_attrs_len:
-	free(attrs_len);
-err_free_scan_element:
-	if (chn->is_scan_element)
-		free(scan_element);
-	return NULL;
+	return alen + ret;
 }
 
 const char * iio_channel_get_id(const struct iio_channel *chn)
@@ -468,12 +383,9 @@ void free_channel(struct iio_channel *chn)
 		free(chn->attrs[i].name);
 		free(chn->attrs[i].filename);
 	}
-	if (chn->nb_attrs)
-		free(chn->attrs);
-	if (chn->name)
-		free(chn->name);
-	if (chn->id)
-		free(chn->id);
+	free(chn->attrs);
+	free(chn->name);
+	free(chn->id);
 	free(chn);
 }
 
@@ -489,11 +401,7 @@ static void shift_bits(uint8_t *dst, size_t shift, size_t len, bool left)
 	size_t i, shift_bytes = shift / 8;
 	shift %= 8;
 
-#if __BYTE_ORDER__ == __ORDER_LITTLE_ENDIAN__
-	if (!left)
-#else
-	if (left)
-#endif
+	if (is_little_endian() ^ left)
 	{
 		if (shift_bytes) {
 			memmove(dst, dst + shift_bytes, len - shift_bytes);
@@ -501,15 +409,15 @@ static void shift_bits(uint8_t *dst, size_t shift, size_t len, bool left)
 		}
 		if (shift) {
 			for (i = 0; i < len; i++) {
-#if __BYTE_ORDER__ == __ORDER_LITTLE_ENDIAN__
-				dst[i] >>= shift;
-				if (i < len - 1)
-					dst[i] |= dst[i + 1] << (8 - shift);
-#else
-				dst[i] <<= shift;
-				if (i < len - 1)
-					dst[i] |= dst[i + 1] >> (8 - shift);
-#endif
+				if (is_little_endian()) {
+					dst[i] >>= shift;
+					if (i < len - 1)
+						dst[i] |= dst[i + 1] << (8 - shift);
+				} else {
+					dst[i] <<= shift;
+					if (i < len - 1)
+						dst[i] |= dst[i + 1] >> (8 - shift);
+				}
 			}
 		}
 	} else {
@@ -519,15 +427,15 @@ static void shift_bits(uint8_t *dst, size_t shift, size_t len, bool left)
 		}
 		if (shift) {
 			for (i = len; i > 0; i--) {
-#if __BYTE_ORDER__ == __ORDER_LITTLE_ENDIAN__
-				dst[i - 1] <<= shift;
-				if (i > 1)
-					dst[i - 1] |= dst[i - 2] >> (8 - shift);
-#else
-				dst[i - 1] >>= shift;
-				if (i > 1)
-					dst[i - 1] |= dst[i - 2] << (8 - shift);
-#endif
+				if (is_little_endian()) {
+					dst[i - 1] <<= shift;
+					if (i > 1)
+						dst[i - 1] |= dst[i - 2] >> (8 - shift);
+				} else {
+					dst[i - 1] >>= shift;
+					if (i > 1)
+						dst[i - 1] |= dst[i - 2] << (8 - shift);
+				}
 			}
 		}
 	}
@@ -538,22 +446,22 @@ static void sign_extend(uint8_t *dst, size_t bits, size_t len)
 	size_t upper_bytes = ((len * 8 - bits) / 8);
 	uint8_t msb, msb_bit = 1 << ((bits - 1) % 8);
 
-#if __BYTE_ORDER__ == __ORDER_LITTLE_ENDIAN__
-	msb = dst[len - 1 - upper_bytes] & msb_bit;
-	if (upper_bytes)
-		memset(dst + len - upper_bytes, msb ? 0xff : 0x00, upper_bytes);
-	if (msb)
-		dst[len - 1 - upper_bytes] |= ~(msb_bit - 1);
-	else
-		dst[len - 1 - upper_bytes] &= (msb_bit - 1);
-#else
-	/* XXX: untested */
-	msb = dst[upper_bytes] & msb_bit;
-	if (upper_bytes)
-		memset(dst, msb ? 0xff : 0x00, upper_bytes);
-	if (msb)
-		dst[upper_bytes] |= ~(msb_bit - 1);
-#endif
+	if (is_little_endian()) {
+		msb = dst[len - 1 - upper_bytes] & msb_bit;
+		if (upper_bytes)
+			memset(dst + len - upper_bytes, msb ? 0xff : 0x00, upper_bytes);
+		if (msb)
+			dst[len - 1 - upper_bytes] |= ~(msb_bit - 1);
+		else
+			dst[len - 1 - upper_bytes] &= (msb_bit - 1);
+	} else {
+		/* XXX: untested */
+		msb = dst[upper_bytes] & msb_bit;
+		if (upper_bytes)
+			memset(dst, msb ? 0xff : 0x00, upper_bytes);
+		if (msb)
+			dst[upper_bytes] |= ~(msb_bit - 1);
+	}
 }
 
 static void mask_upper_bits(uint8_t *dst, size_t bits, size_t len)
@@ -577,11 +485,7 @@ void iio_channel_convert(const struct iio_channel *chn,
 	unsigned int len = chn->format.length / 8;
 	ptrdiff_t end = len * chn->format.repeat;
 	uintptr_t end_ptr = src_ptr + end;
-#if __BYTE_ORDER__ == __ORDER_LITTLE_ENDIAN__
-	bool swap = chn->format.is_be;
-#else
-	bool swap = !chn->format.is_be;
-#endif
+	bool swap = is_little_endian() ^ !chn->format.is_be;
 
 	for (src_ptr = (uintptr_t) src; src_ptr < end_ptr;
 			src_ptr += len, dst_ptr += len) {
@@ -613,11 +517,7 @@ void iio_channel_convert_inverse(const struct iio_channel *chn,
 	unsigned int len = chn->format.length / 8;
 	ptrdiff_t end = len * chn->format.repeat;
 	uintptr_t end_ptr = dst_ptr + end;
-#if __BYTE_ORDER__ == __ORDER_LITTLE_ENDIAN__
-	bool swap = chn->format.is_be;
-#else
-	bool swap = !chn->format.is_be;
-#endif
+	bool swap = is_little_endian() ^ !chn->format.is_be;
 	uint8_t buf[1024];
 
 	/* Somehow I doubt we will have samples of 8192 bits each. */
