@@ -1,18 +1,9 @@
+// SPDX-License-Identifier: LGPL-2.1-or-later
 /*
  * libiio - Library for interfacing industrial I/O (IIO) devices
  *
  * Copyright (C) 2016 Analog Devices, Inc.
  * Author: Paul Cercueil <paul.cercueil@analog.com>
- *
- * This library is free software; you can redistribute it and/or
- * modify it under the terms of the GNU Lesser General Public
- * License as published by the Free Software Foundation; either
- * version 2.1 of the License, or (at your option) any later version.
- *
- * This library is distributed in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
- * Lesser General Public License for more details.
  */
 
 #include "iio-config.h"
@@ -23,12 +14,8 @@
 #include <string.h>
 
 struct iio_scan_context {
-#ifdef WITH_USB_BACKEND
-	struct iio_scan_backend_context *usb_ctx;
-#endif
-#ifdef HAVE_DNS_SD
-	struct iio_scan_backend_context *dnssd_ctx;
-#endif
+	bool scan_usb;
+	bool scan_network;
 	bool scan_local;
 };
 
@@ -49,8 +36,7 @@ ssize_t iio_scan_context_get_info_list(struct iio_scan_context *ctx,
 {
 	struct iio_scan_result scan_result = { 0, NULL };
 
-#ifdef WITH_LOCAL_BACKEND
-	if (ctx->scan_local) {
+	if (WITH_LOCAL_BACKEND && ctx->scan_local) {
 		int ret = local_context_scan(&scan_result);
 		if (ret < 0) {
 			if (scan_result.info)
@@ -58,29 +44,24 @@ ssize_t iio_scan_context_get_info_list(struct iio_scan_context *ctx,
 			return ret;
 		}
 	}
-#endif
 
-#ifdef WITH_USB_BACKEND
-	if (ctx->usb_ctx) {
-		int ret = usb_context_scan(ctx->usb_ctx, &scan_result);
+	if (WITH_USB_BACKEND && ctx->scan_usb) {
+		int ret = usb_context_scan(&scan_result);
 		if (ret < 0) {
 			if (scan_result.info)
 				iio_context_info_list_free(scan_result.info);
 			return ret;
 		}
 	}
-#endif
 
-#ifdef HAVE_DNS_SD
-	if (ctx->dnssd_ctx) {
-		int ret = dnssd_context_scan(ctx->dnssd_ctx, &scan_result);
+	if (HAVE_DNS_SD && ctx->scan_network) {
+		int ret = dnssd_context_scan(&scan_result);
 		if (ret < 0) {
 			if (scan_result.info)
 				iio_context_info_list_free(scan_result.info);
 			return ret;
 		}
 	}
-#endif
 
 	*info = scan_result.info;
 
@@ -97,43 +78,35 @@ void iio_context_info_list_free(struct iio_context_info **list)
 	for (it = list; *it; it++) {
 		struct iio_context_info *info = *it;
 
-		if (info->description)
-			free(info->description);
-		if (info->uri)
-			free(info->uri);
+		free(info->description);
+		free(info->uri);
 		free(info);
 	}
 
 	free(list);
 }
 
-struct iio_context_info ** iio_scan_result_add(
-		struct iio_scan_result *scan_result, size_t num)
+struct iio_context_info *
+iio_scan_result_add(struct iio_scan_result *scan_result)
 {
 	struct iio_context_info **info;
-	size_t old_size, new_size;
-	size_t i;
+	size_t size = scan_result->size;
 
-	old_size = scan_result->size;
-	new_size = old_size + num;
-
-	info = realloc(scan_result->info, (new_size + 1) * sizeof(*info));
+	info = realloc(scan_result->info, (size + 2) * sizeof(*info));
 	if (!info)
 		return NULL;
 
 	scan_result->info = info;
-	scan_result->size = new_size;
+	scan_result->size = size + 1;
 
-	for (i = old_size; i < new_size; i++) {
-		/* Make sure iio_context_info_list_free won't overflow */
-		info[i + 1] = NULL;
+	/* Make sure iio_context_info_list_free won't overflow */
+	info[size + 1] = NULL;
 
-		info[i] = zalloc(sizeof(**info));
-		if (!info[i])
-			return NULL;
-	}
+	info[size] = zalloc(sizeof(**info));
+	if (!info[size])
+		return NULL;
 
-	return &info[old_size];
+	return info[size];
 }
 
 struct iio_scan_context * iio_create_scan_context(
@@ -156,28 +129,17 @@ struct iio_scan_context * iio_create_scan_context(
 	if (!backend || strstr(backend, "local"))
 		ctx->scan_local = true;
 
-#ifdef WITH_USB_BACKEND
 	if (!backend || strstr(backend, "usb"))
-		ctx->usb_ctx = usb_context_scan_init();
-#endif
-#ifdef HAVE_DNS_SD
+		ctx->scan_usb = true;
+
 	if (!backend || strstr(backend, "ip"))
-		ctx->dnssd_ctx = dnssd_context_scan_init();
-#endif
+		ctx->scan_network = true;
 
 	return ctx;
 }
 
 void iio_scan_context_destroy(struct iio_scan_context *ctx)
 {
-#ifdef WITH_USB_BACKEND
-	if (ctx->usb_ctx)
-		usb_context_scan_free(ctx->usb_ctx);
-#endif
-#ifdef HAVE_DNS_SD
-	if (ctx->dnssd_ctx)
-		dnssd_context_scan_free(ctx->dnssd_ctx);
-#endif
 	free(ctx);
 }
 
