@@ -99,21 +99,31 @@ eek:
 	return ret;
 }
 
+static inline const char *get_label_or_name_or_id(const struct iio_device *dev)
+{
+	const char *label, *name;
+
+	label = iio_device_get_label(dev);
+	if (label)
+		return label;
+
+	name = iio_device_get_name(dev);
+	if (name)
+		return name;
+
+	return iio_device_get_id(dev);
+}
+
 static int dump_device_attributes(const struct iio_device *dev,
 		const char *attr, const char *wbuf, enum verbosity quiet)
 {
 	ssize_t ret = 0;
 	char *buf = xmalloc(BUF_SIZE, MY_NAME);
-	const char *name = iio_device_get_name(dev);
 
 	if (!wbuf || quiet == ATTR_VERBOSE) {
 		if (quiet == ATTR_VERBOSE) {
 			printf("%s ", iio_device_is_trigger(dev) ? "trig" : "dev");
-			if(name)
-				printf("'%s'", name);
-			else
-				printf("'%s'", iio_device_get_id(dev));
-
+			printf("'%s'", get_label_or_name_or_id(dev));
 			printf(", attr '%s', value :", attr);
 		}
 		gen_function("device", "dev", attr, NULL);
@@ -125,7 +135,7 @@ static int dump_device_attributes(const struct iio_device *dev,
 				printf("'%s'\n", buf);
 		} else {
 			iio_strerror(-(int)ret, buf, BUF_SIZE);
-			printf("ERROR: %s (%li)\n", buf, (long)ret);
+			printf("ERROR: %s\n", buf);
 		}
 	}
 	if (wbuf) {
@@ -137,8 +147,8 @@ static int dump_device_attributes(const struct iio_device *dev,
 			dump_device_attributes(dev, attr, NULL, quiet);
 		} else {
 			iio_strerror(-(int)ret, buf, BUF_SIZE);
-			printf("ERROR: %s (%li) while writing '%s' with '%s'\n",
-					buf, (long)ret, attr, wbuf);
+			printf("ERROR: %s while writing '%s' with '%s'\n",
+					buf, attr, wbuf);
 		}
 	}
 	free(buf);
@@ -151,7 +161,6 @@ static int dump_buffer_attributes(const struct iio_device *dev,
 {
 	ssize_t ret = 0;
 	char *buf = xmalloc(BUF_SIZE, MY_NAME);
-	const char *name = iio_device_get_name(dev);
 
 	if (!wbuf || quiet == ATTR_VERBOSE ) {
 		gen_function("device_buffer", "dev", attr, NULL);
@@ -159,11 +168,7 @@ static int dump_buffer_attributes(const struct iio_device *dev,
 
 		if (quiet == ATTR_VERBOSE) {
 			printf("%s ", iio_device_is_trigger(dev) ? "trig" : "dev");
-			if (name)
-				printf("'%s'", name);
-			else
-				printf("'%s'", iio_device_get_id(dev));
-
+			printf("'%s'", get_label_or_name_or_id(dev));
 			printf(", buffer attr '%s', value :", attr);
 		}
 
@@ -174,7 +179,7 @@ static int dump_buffer_attributes(const struct iio_device *dev,
 				printf("'%s'\n", buf);
 		} else {
 			iio_strerror(-(int)ret, buf, BUF_SIZE);
-			printf("ERROR: %s (%li)\n", buf, (long)ret);
+			printf("ERROR: %s\n", buf);
 		}
 	}
 
@@ -187,8 +192,8 @@ static int dump_buffer_attributes(const struct iio_device *dev,
 			dump_buffer_attributes(dev, attr, NULL, quiet);
 		} else {
 			iio_strerror(-(int)ret, buf, BUF_SIZE);
-			printf("ERROR: %s (%li) while writing '%s' with '%s'\n",
-					buf, (long)ret, attr, wbuf);
+			printf("ERROR: %s while writing '%s' with '%s'\n",
+					buf, attr, wbuf);
 		}
 	}
 
@@ -201,7 +206,6 @@ static int dump_debug_attributes(const struct iio_device *dev,
 {
 	ssize_t ret = 0;
 	char *buf = xmalloc(BUF_SIZE, MY_NAME);
-	const char *name = iio_device_get_name(dev);
 
 	if (!wbuf || quiet == ATTR_VERBOSE) {
 		gen_function("device_debug", "dev", attr, NULL);
@@ -209,11 +213,7 @@ static int dump_debug_attributes(const struct iio_device *dev,
 
 		if (quiet == ATTR_VERBOSE) {
 			printf("%s ", iio_device_is_trigger(dev) ? "trig" : "dev");
-			if (name)
-				printf("'%s'", name);
-			else
-				printf("'%s'", iio_device_get_id(dev));
-
+			printf("'%s'", get_label_or_name_or_id(dev));
 			printf(", debug attr '%s', value :", attr);
 
 		}
@@ -225,7 +225,7 @@ static int dump_debug_attributes(const struct iio_device *dev,
 				printf("'%s'\n", buf);
 		} else {
 			iio_strerror(-(int)ret, buf, BUF_SIZE);
-			printf("ERROR: %s (%li)\n", buf, (long)ret);
+			printf("ERROR: %s\n", buf);
 		}
 	}
 
@@ -238,8 +238,8 @@ static int dump_debug_attributes(const struct iio_device *dev,
 			dump_debug_attributes(dev, attr, NULL, quiet);
 		} else {
 			iio_strerror(-(int)ret, buf, BUF_SIZE);
-			printf("ERROR: %s (%li) while writing '%s' with '%s'\n",
-					buf, (long)ret, attr, wbuf);
+			printf("ERROR: %s while writing '%s' with '%s'\n",
+					buf, attr, wbuf);
 		}
 	}
 
@@ -253,7 +253,6 @@ static int dump_channel_attributes(const struct iio_device *dev,
 	ssize_t ret = 0;
 	char *buf = xmalloc(BUF_SIZE, MY_NAME);
 	const char *type_name;
-	const char *name = iio_device_get_name(dev);
 
 	if (!wbuf || quiet == ATTR_VERBOSE) {
 		if (iio_channel_is_output(ch))
@@ -265,11 +264,7 @@ static int dump_channel_attributes(const struct iio_device *dev,
 		ret = iio_channel_attr_read(ch, attr, buf, BUF_SIZE);
 		if (quiet == ATTR_VERBOSE) {
 			printf("%s ", iio_device_is_trigger(dev) ? "trig" : "dev");
-			if (name)
-				printf("'%s'", name);
-			else
-				printf("'%s'", iio_device_get_id(dev));
-
+			printf("'%s'", get_label_or_name_or_id(dev));
 			printf(", channel '%s' (%s), ",
 					iio_channel_get_id(ch),
 					type_name);
@@ -287,7 +282,7 @@ static int dump_channel_attributes(const struct iio_device *dev,
 				printf("value '%s'\n", buf);
 		} else {
 			iio_strerror(-(int)ret, buf, BUF_SIZE);
-			printf("ERROR: %s (%li)\n", buf, (long)ret);
+			printf("ERROR: %s\n", buf);
 		}
 	}
 	if (wbuf) {
@@ -299,8 +294,8 @@ static int dump_channel_attributes(const struct iio_device *dev,
 			dump_channel_attributes(dev, ch, attr, NULL, quiet);
 		} else {
 			iio_strerror(-(int)ret, buf, BUF_SIZE);
-			printf("error %s (%li) while writing '%s' with '%s'\n",
-					buf, (long)ret, attr, wbuf);
+			printf("error %s while writing '%s' with '%s'\n",
+					buf, attr, wbuf);
 		}
 	}
 	free(buf);
@@ -353,7 +348,7 @@ int main(int argc, char **argv)
 {
 	char **argw;
 	struct iio_context *ctx;
-	int c;
+	int c, argd = argc;
 	int device_index = 0, channel_index = 0, attr_index = 0;
 	const char *gen_file = NULL;
 	bool search_device = false, ignore_case = false,
@@ -371,13 +366,24 @@ int main(int argc, char **argv)
 
 	argw = dup_argv(MY_NAME, argc, argv);
 
-	ctx = handle_common_opts(MY_NAME, argc, argw, MY_OPTS, options, options_descriptions);
+	/*
+	 * getopt_long() thinks negative numbers are options, -1 is option '1'
+	 * The only time we should see a negative number is the last argument during a write,
+	 * so if there is one, we skip that argument during getopt processing
+	 * look for "-" followed by a number.
+	 */
+	if (strnlen(argv[argc - 1], 2) >= 2 && argv[argc - 1][0] == '-' && 
+			(argv[argc - 1][1] >= '0' && argv[argc - 1][1] <= '9')) {
+		argd--;
+	}
+
+	ctx = handle_common_opts(MY_NAME, argd, argw, MY_OPTS, options, options_descriptions);
 	opts = add_common_options(options);
 	if (!opts) {
 		fprintf(stderr, "Failed to add common options\n");
 		return EXIT_FAILURE;
 	}
-	while ((c = getopt_long(argc, argw, "+" COMMON_OPTIONS MY_OPTS, /* Flawfinder: ignore */
+	while ((c = getopt_long(argd, argw, "+" COMMON_OPTIONS MY_OPTS, /* Flawfinder: ignore */
 					opts, NULL)) != -1) {
 		switch (c) {
 		/* All these are handled in the common */
@@ -385,6 +391,7 @@ int main(int argc, char **argv)
 		case 'n':
 		case 'x':
 		case 'u':
+		case 'T':
 			break;
 		case 'S':
 			context_scan = true;
@@ -618,8 +625,8 @@ int main(int argc, char **argv)
 			} else {
 				char *buf = xmalloc(BUF_SIZE, MY_NAME);
 				iio_strerror(errno, buf, BUF_SIZE);
-				fprintf(stderr, "Unable to get context attributes: %s (%zd)\n",
-						buf, ret);
+				fprintf(stderr, "Unable to get context attributes: %s\n",
+						buf);
 				free(buf);
 			}
 		}
@@ -633,12 +640,16 @@ int main(int argc, char **argv)
 
 		for (i = 0; i < nb_devices; i++) {
 			const struct iio_device *dev = iio_context_get_device(ctx, i);
-			const char *name = iio_device_get_name(dev);
-			const char *ch_name;
 			const char *dev_id = iio_device_get_id(dev);
+			const char *label = iio_device_get_label(dev);
+			const char *name = iio_device_get_name(dev);
+			const char *label_or_name = label ? label : name;
+			const char *label_or_name_or_id = label_or_name ? label_or_name : dev_id;
+			const char *ch_name;
 			unsigned int nb_attrs, nb_channels, j;
 
 			if (device_index && !str_match(dev_id, argw[device_index], ignore_case)
+					&& !str_match(label, argw[device_index], ignore_case)
 					&& !str_match(name, argw[device_index], ignore_case)) {
 				continue;
 			}
@@ -647,8 +658,8 @@ int main(int argc, char **argv)
 			if ((search_device && !device_index) || (search_channel && !device_index) ||
 					(search_buffer && !device_index) || (search_debug && !device_index)) {
 				printf("\t%s", dev_id);
-				if (name)
-					printf(", %s", name);
+				if (label_or_name)
+					printf(", %s", label_or_name);
 				printf(": ");
 			}
 
@@ -724,10 +735,7 @@ int main(int argc, char **argv)
 				if ((!scan_only && !channel_index) ||
 				    ( scan_only && iio_channel_is_scan_element(ch))) {
 					printf("%s ", iio_device_is_trigger(dev) ? "trig" : "dev");
-					if (name)
-						printf("'%s', ", name);
-					else
-						printf("'%s', ", dev_id);
+					printf("'%s', ", label_or_name_or_id);
 
 					printf("channel '%s'",
 						iio_channel_get_id(ch));
@@ -806,7 +814,7 @@ int main(int argc, char **argv)
 				printf("found %u device attributes\n", nb_attrs);
 			if (search_device && device_index && !attr_index && !nb_attrs) {
 				printf("%s: Found %s device, but it has %u device attributes\n",
-						MY_NAME, name ? name : dev_id, nb_attrs);
+						MY_NAME, label_or_name_or_id, nb_attrs);
 				if (!attr_found)
 					found_err = true;
 			}
@@ -838,7 +846,7 @@ int main(int argc, char **argv)
 				printf("found %u buffer attributes\n", nb_attrs);
 			if (search_buffer && device_index && !attr_index && !nb_attrs) {
 				printf("%s: Found %s device, but it has %u buffer attributes\n",
-						MY_NAME, name ? name : dev_id, nb_attrs);
+						MY_NAME, label_or_name_or_id, nb_attrs);
 				if (!attr_found)
 					found_err = true;
 			}
