@@ -95,6 +95,7 @@ int main(int argc, char **argv)
 		case 'n':
 		case 'x':
 		case 'u':
+		case 'T':
 			break;
 		case 'S':
 		case 'a':
@@ -131,7 +132,7 @@ int main(int argc, char **argv)
 	else {
 		char err_str[1024];
 		iio_strerror(-ret, err_str, sizeof(err_str));
-		fprintf(stderr, "Unable to get backend version: %s (%i)\n", err_str, ret);
+		fprintf(stderr, "Unable to get backend version: %s\n", err_str);
 	}
 
 	printf("Backend description string: %s\n",
@@ -150,8 +151,8 @@ int main(int argc, char **argv)
 		else {
 			char err_str[1024];
 			iio_strerror(-ret, err_str, sizeof(err_str));
-			fprintf(stderr, "\tUnable to read IIO context attributes: %s (%i)\n",
-					err_str, ret);
+			fprintf(stderr, "\tUnable to read IIO context attributes: %s\n",
+					err_str);
 		}
 	}
 
@@ -162,9 +163,12 @@ int main(int argc, char **argv)
 	for (i = 0; i < nb_devices; i++) {
 		const struct iio_device *dev = iio_context_get_device(ctx, i);
 		const char *name = iio_device_get_name(dev);
+		const char *label = iio_device_get_label(dev);
 		printf("\t%s:", iio_device_get_id(dev));
 		if (name)
 			printf(" %s", name);
+		if (label)
+			printf(" (label: %s)", label);
 		if (dev_is_buffer_capable(dev))
 			printf(" (buffer capable)");
 		printf("\n");
@@ -232,7 +236,7 @@ int main(int argc, char **argv)
 					printf("value: %s\n", buf);
 				} else {
 					iio_strerror(-ret, buf, BUF_SIZE);
-					printf("ERROR: %s (%i)\n", buf, ret);
+					printf("ERROR: %s\n", buf);
 				}
 			}
 		}
@@ -253,7 +257,7 @@ int main(int argc, char **argv)
 					printf("value: %s\n", buf);
 				} else {
 					iio_strerror(-ret, buf, BUF_SIZE);
-					printf("ERROR: %s (%i)\n", buf, ret);
+					printf("ERROR: %s\n", buf);
 				}
 			}
 		}
@@ -274,7 +278,7 @@ int main(int argc, char **argv)
 					printf("value: %s\n", buf);
 				} else {
 					iio_strerror(-ret, buf, BUF_SIZE);
-					printf("ERROR: %s (%i)\n", buf, ret);
+					printf("ERROR: %s\n", buf);
 				}
 			}
 		}
@@ -294,7 +298,7 @@ int main(int argc, char **argv)
 					printf("value: %s\n", buf);
 				} else {
 					iio_strerror(-ret, buf, BUF_SIZE);
-					printf("ERROR: %s (%i)\n", buf, ret);
+					printf("ERROR: %s\n", buf);
 				}
 			}
 		}
@@ -314,7 +318,7 @@ int main(int argc, char **argv)
 			printf("\t\tNo trigger on this device\n");
 		} else if (ret < 0) {
 			iio_strerror(-ret, buf, BUF_SIZE);
-			printf("ERROR: checking for trigger : %s (%i)\n", buf, ret);
+			printf("ERROR: checking for trigger : %s\n", buf);
 		}
 	}
 
