@@ -1,20 +1,10 @@
+// SPDX-License-Identifier: LGPL-2.1-or-later
 /*
  * libiio - Library for interfacing industrial I/O (IIO) devices
  *
  * Copyright (C) 2015 - 2020 Analog Devices, Inc.
  * Author: Paul Cercueil <paul.cercueil@analog.com>
- *
- * This library is free software; you can redistribute it and/or
- * modify it under the terms of the GNU Lesser General Public
- * License as published by the Free Software Foundation; either
- * version 2.1 of the License, or (at your option) any later version.
- *
- * This library is distributed in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
- * Lesser General Public License for more details.
- *
- * */
+ */
 
 #include "iio-lock.h"
 #include "iio-private.h"
@@ -26,10 +16,6 @@
 #include <stdbool.h>
 #include <string.h>
 
-#ifdef ERROR
-#undef ERROR
-#endif
-
 #include "debug.h"
 
 #define DEFAULT_TIMEOUT_MS 5000
@@ -47,7 +33,7 @@ struct iio_usb_ep_couple {
 	struct iio_mutex *lock;
 };
 
-struct iio_usb_io_context {
+struct iiod_client_pdata {
 	struct iio_usb_ep_couple *ep;
 
 	struct iio_mutex *lock;
@@ -62,9 +48,6 @@ struct iio_context_pdata {
 
 	struct iiod_client *iiod_client;
 
-	/* Lock for non-streaming operations */
-	struct iio_mutex *lock;
-
 	/* Lock for endpoint reservation */
 	struct iio_mutex *ep_lock;
 
@@ -73,14 +56,14 @@ struct iio_context_pdata {
 
 	unsigned int timeout_ms;
 
-	struct iio_usb_io_context io_ctx;
+	struct iiod_client_pdata io_ctx;
 };
 
 struct iio_device_pdata {
 	struct iio_mutex *lock;
 
 	bool opened;
-	struct iio_usb_io_context io_ctx;
+	struct iiod_client_pdata io_ctx;
 };
 
 static const unsigned int libusb_to_errno_codes[] = {
@@ -119,7 +102,7 @@ static unsigned int libusb_to_errno(int error)
 	}
 }
 
-static int usb_io_context_init(struct iio_usb_io_context *io_ctx)
+static int usb_io_context_init(struct iiod_client_pdata *io_ctx)
 {
 	io_ctx->lock = iio_mutex_create();
 	if (!io_ctx->lock)
@@ -128,7 +111,7 @@ static int usb_io_context_init(struct iio_usb_io_context *io_ctx)
 	return 0;
 }
 
-static void usb_io_context_exit(struct iio_usb_io_context *io_ctx)
+static void usb_io_context_exit(struct iiod_client_pdata *io_ctx)
 {
 	if (io_ctx->lock) {
 		iio_mutex_destroy(io_ctx->lock);
@@ -139,8 +122,10 @@ static void usb_io_context_exit(struct iio_usb_io_context *io_ctx)
 static int usb_get_version(const struct iio_context *ctx,
 		unsigned int *major, unsigned int *minor, char git_tag[8])
 {
-	return iiod_client_get_version(ctx->pdata->iiod_client,
-			&ctx->pdata->io_ctx, major, minor, git_tag);
+	struct iio_context_pdata *pdata = iio_context_get_pdata(ctx);
+
+	return iiod_client_get_version(pdata->iiod_client,
+			&pdata->io_ctx, major, minor, git_tag);
 }
 
 static unsigned int usb_calculate_remote_timeout(unsigned int timeout)
@@ -223,7 +208,7 @@ static int usb_close_pipe(struct iio_context_pdata *pdata, uint16_t pipe_id)
 
 static int usb_reserve_ep_unlocked(const struct iio_device *dev)
 {
-	struct iio_context_pdata *pdata = dev->ctx->pdata;
+	struct iio_context_pdata *pdata = iio_context_get_pdata(dev->ctx);
 	unsigned int i;
 
 	for (i = 0; i < pdata->nb_ep_couples; i++) {
@@ -243,7 +228,7 @@ static int usb_reserve_ep_unlocked(const struct iio_device *dev)
 
 static void usb_free_ep_unlocked(const struct iio_device *dev)
 {
-	struct iio_context_pdata *pdata = dev->ctx->pdata;
+	struct iio_context_pdata *pdata = iio_context_get_pdata(dev->ctx);
 	unsigned int i;
 
 	for (i = 0; i < pdata->nb_ep_couples; i++) {
@@ -259,7 +244,7 @@ static void usb_free_ep_unlocked(const struct iio_device *dev)
 static int usb_open(const struct iio_device *dev,
 		size_t samples_count, bool cyclic)
 {
-	struct iio_context_pdata *ctx_pdata = dev->ctx->pdata;
+	struct iio_context_pdata *ctx_pdata = iio_context_get_pdata(dev->ctx);
 	struct iio_device_pdata *pdata = dev->pdata;
 	int ret = -EBUSY;
 
@@ -313,7 +298,7 @@ out_unlock:
 
 static int usb_close(const struct iio_device *dev)
 {
-	struct iio_context_pdata *ctx_pdata = dev->ctx->pdata;
+	struct iio_context_pdata *ctx_pdata = iio_context_get_pdata(dev->ctx);
 	struct iio_device_pdata *pdata = dev->pdata;
 	int ret = -EBADF;
 
@@ -340,11 +325,12 @@ out_unlock:
 static ssize_t usb_read(const struct iio_device *dev, void *dst, size_t len,
 		uint32_t *mask, size_t words)
 {
+	struct iio_context_pdata *ctx_pdata = iio_context_get_pdata(dev->ctx);
 	struct iio_device_pdata *pdata = dev->pdata;
 	ssize_t ret;
 
 	iio_mutex_lock(pdata->lock);
-	ret = iiod_client_read_unlocked(dev->ctx->pdata->iiod_client,
+	ret = iiod_client_read_unlocked(ctx_pdata->iiod_client,
 			&pdata->io_ctx, dev, dst, len, mask, words);
 	iio_mutex_unlock(pdata->lock);
 
@@ -354,11 +340,12 @@ static ssize_t usb_read(const struct iio_device *dev, void *dst, size_t len,
 static ssize_t usb_write(const struct iio_device *dev,
 		const void *src, size_t len)
 {
+	struct iio_context_pdata *ctx_pdata = iio_context_get_pdata(dev->ctx);
 	struct iio_device_pdata *pdata = dev->pdata;
 	ssize_t ret;
 
 	iio_mutex_lock(pdata->lock);
-	ret = iiod_client_write_unlocked(dev->ctx->pdata->iiod_client,
+	ret = iiod_client_write_unlocked(ctx_pdata->iiod_client,
 			&pdata->io_ctx, dev, src, len);
 	iio_mutex_unlock(pdata->lock);
 
@@ -368,7 +355,7 @@ static ssize_t usb_write(const struct iio_device *dev,
 static ssize_t usb_read_dev_attr(const struct iio_device *dev,
 		const char *attr, char *dst, size_t len, enum iio_attr_type type)
 {
-	struct iio_context_pdata *pdata = dev->ctx->pdata;
+	struct iio_context_pdata *pdata = iio_context_get_pdata(dev->ctx);
 
 	return iiod_client_read_attr(pdata->iiod_client,
 			&pdata->io_ctx, dev, NULL, attr,
@@ -378,7 +365,7 @@ static ssize_t usb_read_dev_attr(const struct iio_device *dev,
 static ssize_t usb_write_dev_attr(const struct iio_device *dev,
 		const char *attr, const char *src, size_t len, enum iio_attr_type type)
 {
-	struct iio_context_pdata *pdata = dev->ctx->pdata;
+	struct iio_context_pdata *pdata = iio_context_get_pdata(dev->ctx);
 
 	return iiod_client_write_attr(pdata->iiod_client,
 			&pdata->io_ctx, dev, NULL, attr,
@@ -388,7 +375,7 @@ static ssize_t usb_write_dev_attr(const struct iio_device *dev,
 static ssize_t usb_read_chn_attr(const struct iio_channel *chn,
 		const char *attr, char *dst, size_t len)
 {
-	struct iio_context_pdata *pdata = chn->dev->ctx->pdata;
+	struct iio_context_pdata *pdata = iio_context_get_pdata(chn->dev->ctx);
 
 	return iiod_client_read_attr(pdata->iiod_client,
 			&pdata->io_ctx, chn->dev, chn, attr,
@@ -398,7 +385,7 @@ static ssize_t usb_read_chn_attr(const struct iio_channel *chn,
 static ssize_t usb_write_chn_attr(const struct iio_channel *chn,
 		const char *attr, const char *src, size_t len)
 {
-	struct iio_context_pdata *pdata = chn->dev->ctx->pdata;
+	struct iio_context_pdata *pdata = iio_context_get_pdata(chn->dev->ctx);
 
 	return iiod_client_write_attr(pdata->iiod_client,
 			&pdata->io_ctx, chn->dev, chn, attr,
@@ -408,7 +395,7 @@ static ssize_t usb_write_chn_attr(const struct iio_channel *chn,
 static int usb_set_kernel_buffers_count(const struct iio_device *dev,
 		unsigned int nb_blocks)
 {
-	struct iio_context_pdata *pdata = dev->ctx->pdata;
+	struct iio_context_pdata *pdata = iio_context_get_pdata(dev->ctx);
 
 	return iiod_client_set_kernel_buffers_count(pdata->iiod_client,
 			&pdata->io_ctx, dev, nb_blocks);
@@ -416,7 +403,7 @@ static int usb_set_kernel_buffers_count(const struct iio_device *dev,
 
 static int usb_set_timeout(struct iio_context *ctx, unsigned int timeout)
 {
-	struct iio_context_pdata *pdata = ctx->pdata;
+	struct iio_context_pdata *pdata = iio_context_get_pdata(ctx);
 	unsigned int remote_timeout = usb_calculate_remote_timeout(timeout);
 	int ret;
 
@@ -431,7 +418,7 @@ static int usb_set_timeout(struct iio_context *ctx, unsigned int timeout)
 static int usb_get_trigger(const struct iio_device *dev,
                 const struct iio_device **trigger)
 {
-	struct iio_context_pdata *pdata = dev->ctx->pdata;
+	struct iio_context_pdata *pdata = iio_context_get_pdata(dev->ctx);
 
 	return iiod_client_get_trigger(pdata->iiod_client,
 			&pdata->io_ctx, dev, trigger);
@@ -440,7 +427,7 @@ static int usb_get_trigger(const struct iio_device *dev,
 static int usb_set_trigger(const struct iio_device *dev,
                 const struct iio_device *trigger)
 {
-	struct iio_context_pdata *pdata = dev->ctx->pdata;
+	struct iio_context_pdata *pdata = iio_context_get_pdata(dev->ctx);
 
 	return iiod_client_set_trigger(pdata->iiod_client,
 			&pdata->io_ctx, dev, trigger);
@@ -449,36 +436,35 @@ static int usb_set_trigger(const struct iio_device *dev,
 
 static void usb_shutdown(struct iio_context *ctx)
 {
+	struct iio_context_pdata *pdata = iio_context_get_pdata(ctx);
+	unsigned int nb_devices = iio_context_get_devices_count(ctx);
 	unsigned int i;
 
-	usb_io_context_exit(&ctx->pdata->io_ctx);
+	usb_io_context_exit(&pdata->io_ctx);
 
-	for (i = 0; i < ctx->nb_devices; i++)
-		usb_close(ctx->devices[i]);
+	for (i = 0; i < nb_devices; i++)
+		usb_close(iio_context_get_device(ctx, i));
 
-	iio_mutex_destroy(ctx->pdata->lock);
-	iio_mutex_destroy(ctx->pdata->ep_lock);
+	iio_mutex_destroy(pdata->ep_lock);
 
-	for (i = 0; i < ctx->pdata->nb_ep_couples; i++)
-		if (ctx->pdata->io_endpoints[i].lock)
-			iio_mutex_destroy(ctx->pdata->io_endpoints[i].lock);
-	if (ctx->pdata->io_endpoints)
-		free(ctx->pdata->io_endpoints);
+	for (i = 0; i < pdata->nb_ep_couples; i++)
+		if (pdata->io_endpoints[i].lock)
+			iio_mutex_destroy(pdata->io_endpoints[i].lock);
+	free(pdata->io_endpoints);
 
-	for (i = 0; i < ctx->nb_devices; i++) {
-		struct iio_device *dev = ctx->devices[i];
+	for (i = 0; i < nb_devices; i++) {
+		struct iio_device *dev = iio_context_get_device(ctx, i);
 
 		usb_io_context_exit(&dev->pdata->io_ctx);
 		free(dev->pdata);
 	}
 
-	iiod_client_destroy(ctx->pdata->iiod_client);
+	iiod_client_destroy(pdata->iiod_client);
 
-	usb_reset_pipes(ctx->pdata); /* Close everything */
+	usb_reset_pipes(pdata); /* Close everything */
 
-	libusb_close(ctx->pdata->hdl);
-	libusb_exit(ctx->pdata->ctx);
-	free(ctx->pdata);
+	libusb_close(pdata->hdl);
+	libusb_exit(pdata->ctx);
 }
 
 static int iio_usb_match_interface(const struct libusb_config_descriptor *desc,
@@ -578,7 +564,7 @@ static void LIBUSB_CALL sync_transfer_cb(struct libusb_transfer *transfer)
 }
 
 static int usb_sync_transfer(struct iio_context_pdata *pdata,
-	struct iio_usb_io_context *io_ctx, unsigned int ep_type,
+	struct iiod_client_pdata *io_ctx, unsigned int ep_type,
 	char *data, size_t len, int *transferred)
 {
 	unsigned char ep;
@@ -683,7 +669,8 @@ unlock:
 }
 
 static ssize_t write_data_sync(struct iio_context_pdata *pdata,
-		void *ep, const char *data, size_t len)
+			       struct iiod_client_pdata *ep,
+			       const char *data, size_t len)
 {
 	int transferred, ret;
 
@@ -696,7 +683,8 @@ static ssize_t write_data_sync(struct iio_context_pdata *pdata,
 }
 
 static ssize_t read_data_sync(struct iio_context_pdata *pdata,
-		void *ep, char *buf, size_t len)
+			      struct iiod_client_pdata *ep,
+			      char *buf, size_t len)
 {
 	int transferred, ret;
 
@@ -740,6 +728,7 @@ static int usb_verify_eps(const struct libusb_interface_descriptor *iface)
 static int usb_populate_context_attrs(struct iio_context *ctx,
 		libusb_device *dev, libusb_device_handle *hdl)
 {
+	struct iio_context_pdata *pdata = iio_context_get_pdata(ctx);
 	struct libusb_device_descriptor dev_desc;
 	char buffer[64];
 	unsigned int i;
@@ -762,7 +751,7 @@ static int usb_populate_context_attrs(struct iio_context *ctx,
 
 	iio_snprintf(uri, sizeof(uri), "usb:%d.%d.%u",
 		libusb_get_bus_number(dev), libusb_get_device_address(dev),
-		(uint8_t)ctx->pdata->intrfc);
+		(uint8_t)pdata->intrfc);
 	ret = iio_context_add_attr(ctx, "uri", uri);
 	if (ret < 0)
 		return ret;
@@ -817,8 +806,8 @@ static int usb_populate_context_attrs(struct iio_context *ctx,
 	return 0;
 }
 
-struct iio_context * usb_create_context(unsigned int bus,
-		uint16_t address, uint16_t intrfc)
+static struct iio_context * usb_create_context(unsigned int bus,
+					       uint16_t address, uint16_t intrfc)
 {
 	libusb_context *usb_ctx;
 	libusb_device_handle *hdl = NULL;
@@ -839,22 +828,14 @@ struct iio_context * usb_create_context(unsigned int bus,
 		goto err_set_errno;
 	}
 
-	pdata->lock = iio_mutex_create();
-	if (!pdata->lock) {
+	pdata->ep_lock = iio_mutex_create();
+	if (!pdata->ep_lock) {
 		IIO_ERROR("Unable to create mutex\n");
 		ret = -ENOMEM;
 		goto err_free_pdata;
 	}
 
-	pdata->ep_lock = iio_mutex_create();
-	if (!pdata->ep_lock) {
-		IIO_ERROR("Unable to create mutex\n");
-		ret = -ENOMEM;
-		goto err_destroy_mutex;
-	}
-
-	pdata->iiod_client = iiod_client_new(pdata, pdata->lock,
-			&usb_iiod_client_ops);
+	pdata->iiod_client = iiod_client_new(pdata, &usb_iiod_client_ops);
 	if (!pdata->iiod_client) {
 		IIO_ERROR("Unable to create IIOD client\n");
 		ret = -errno;
@@ -872,7 +853,7 @@ struct iio_context * usb_create_context(unsigned int bus,
 	if (ret < 0) {
 		ret = -(int) libusb_to_errno(ret);
 		IIO_ERROR("Unable to get usb device list: %i\n", ret);
-		goto err_destroy_iiod_client;
+		goto err_libusb_exit;
 	}
 
 	usb_dev = NULL;
@@ -924,8 +905,8 @@ struct iio_context * usb_create_context(unsigned int bus,
 	if (ret) {
 		ret = -(int) libusb_to_errno(ret);
 		iio_strerror(-ret, err_str, sizeof(err_str));
-		IIO_ERROR("Unable to claim interface %u:%u:%u: %s (%i)\n",
-		      bus, address, intrfc, err_str, ret);
+		IIO_ERROR("Unable to claim interface %u:%u:%u: %s\n",
+		      bus, address, intrfc, err_str);
 		goto err_libusb_close;
 	}
 
@@ -933,8 +914,7 @@ struct iio_context * usb_create_context(unsigned int bus,
 	if (ret) {
 		ret = -(int) libusb_to_errno(ret);
 		iio_strerror(-ret, err_str, sizeof(err_str));
-		IIO_ERROR("Unable to get config descriptor: %s (%i)\n",
-				err_str, ret);
+		IIO_ERROR("Unable to get config descriptor: %s\n", err_str);
 		goto err_libusb_close;
 	}
 
@@ -1015,8 +995,8 @@ struct iio_context * usb_create_context(unsigned int bus,
 	ctx->ops = &usb_ops;
 	ctx->pdata = pdata;
 
-	for (i = 0; i < ctx->nb_devices; i++) {
-		struct iio_device *dev = ctx->devices[i];
+	for (i = 0; i < iio_context_get_devices_count(ctx); i++) {
+		struct iio_device *dev = iio_context_get_device(ctx, i);
 
 		dev->pdata = zalloc(sizeof(*dev->pdata));
 		if (!dev->pdata) {
@@ -1049,8 +1029,7 @@ err_free_endpoints:
 	for (i = 0; i < pdata->nb_ep_couples; i++)
 		if (pdata->io_endpoints[i].lock)
 			iio_mutex_destroy(pdata->io_endpoints[i].lock);
-	if (pdata->io_endpoints)
-		free(pdata->io_endpoints);
+	free(pdata->io_endpoints);
 err_free_config_descriptor:
 	libusb_free_config_descriptor(conf_desc);
 err_libusb_close:
@@ -1061,8 +1040,6 @@ err_destroy_iiod_client:
 	iiod_client_destroy(pdata->iiod_client);
 err_destroy_ep_mutex:
 	iio_mutex_destroy(pdata->ep_lock);
-err_destroy_mutex:
-	iio_mutex_destroy(pdata->lock);
 err_free_pdata:
 	free(pdata);
 err_set_errno:
@@ -1223,49 +1200,24 @@ static int usb_fill_context_info(struct iio_context_info *info,
 	return 0;
 }
 
-struct iio_scan_backend_context {
-	libusb_context *ctx;
-};
-
-struct iio_scan_backend_context * usb_context_scan_init(void)
+int usb_context_scan(struct iio_scan_result *scan_result)
 {
-	struct iio_scan_backend_context *ctx;
-	int ret;
-
-	ctx = malloc(sizeof(*ctx));
-	if (!ctx) {
-		errno = ENOMEM;
-		return NULL;
-	}
-
-	ret = libusb_init(&ctx->ctx);
-	if (ret) {
-		free(ctx);
-		errno = (int) libusb_to_errno(ret);
-		return NULL;
-	}
-
-	return ctx;
-}
-
-void usb_context_scan_free(struct iio_scan_backend_context *ctx)
-{
-	libusb_exit(ctx->ctx);
-	free(ctx);
-}
-
-int usb_context_scan(struct iio_scan_backend_context *ctx,
-		struct iio_scan_result *scan_result)
-{
-	struct iio_context_info **info;
+	struct iio_context_info *info;
 	libusb_device **device_list;
+	libusb_context *ctx;
 	unsigned int i;
 	int ret;
 
-	ret = (int) libusb_get_device_list(ctx->ctx, &device_list);
+	ret = libusb_init(&ctx);
 	if (ret < 0)
 		return -(int) libusb_to_errno(ret);
 
+	ret = (int) libusb_get_device_list(ctx, &device_list);
+	if (ret < 0) {
+		ret = -(int) libusb_to_errno(ret);
+		goto cleanup_libusb_exit;
+	}
+
 	for (i = 0; device_list[i]; i++) {
 		struct libusb_device_handle *hdl;
 		struct libusb_device *dev = device_list[i];
@@ -1276,12 +1228,12 @@ int usb_context_scan(struct iio_scan_backend_context *ctx,
 			continue;
 
 		if (!iio_usb_match_device(dev, hdl, &intrfc)) {
-			info = iio_scan_result_add(scan_result, 1);
+			info = iio_scan_result_add(scan_result);
 			if (!info)
 				ret = -ENOMEM;
 			else
-				ret = usb_fill_context_info(*info, dev, hdl,
-						intrfc);
+				ret = usb_fill_context_info(info, dev, hdl,
+							    intrfc);
 		}
 
 		libusb_close(hdl);
@@ -1293,5 +1245,7 @@ int usb_context_scan(struct iio_scan_backend_context *ctx,
 
 cleanup_free_device_list:
 	libusb_free_device_list(device_list, true);
+cleanup_libusb_exit:
+	libusb_exit(ctx);
 	return ret;
 }
