@@ -1,30 +1,21 @@
+// SPDX-License-Identifier: LGPL-2.1-or-later
 /*
  * libiio - Library for interfacing industrial I/O (IIO) devices
  *
  * Copyright (C) 2014 Analog Devices, Inc.
  * Author: Paul Cercueil <paul.cercueil@analog.com>
- *
- * This library is free software; you can redistribute it and/or
- * modify it under the terms of the GNU Lesser General Public
- * License as published by the Free Software Foundation; either
- * version 2.1 of the License, or (at your option) any later version.
- *
- * This library is distributed in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
- * Lesser General Public License for more details.
- *
- * */
+ */
 
 /* Force the XSI version of strerror_r */
 #undef _GNU_SOURCE
 
+#include "dns_sd.h"
 #include "iio-config.h"
 #include "iio-private.h"
-#include "network.h"
 
 #include <errno.h>
 #include <locale.h>
+#include <stdarg.h>
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
@@ -216,6 +207,10 @@ void iio_strerror(int err, char *buf, size_t len)
 #endif
 	if (ret != 0)
 		iio_snprintf(buf, len, "Unknown error %i", err);
+	else {
+		size_t i = strnlen(buf, len);
+		iio_snprintf(buf + i, len - i, " (%i)", err);
+	}
 }
 
 char *iio_strdup(const char *str)
@@ -301,7 +296,7 @@ char * iio_getenv (char * envvar)
 	len = strnlen(hostname, tmp);
 
 	/* Should be smaller than max length */
-	if (len <= tmp)
+	if (len == tmp)
 		goto wrong_str;
 
 	/* should be more than "usb:" or "ip:" */
@@ -321,3 +316,17 @@ wrong_str:
 #endif
 	return NULL;
 }
+
+ssize_t __iio_printf iio_snprintf(char *buf, size_t len, const char *fmt, ...)
+{
+	va_list ap;
+	int ret;
+
+	va_start(ap, fmt);
+	ret = vsnprintf(buf, len, fmt, ap);
+	if (len && ret >= (ssize_t)len)
+		ret = -ERANGE;
+	va_end(ap);
+
+	return (ssize_t)ret;
+}
